#!/bin/bash

H2O_JAR_NAME=h2o-writedelay.jar

WORKER_DEBUG=false
COORD_DEBUG=false


if [ $# != "3" ]; then
	echo "Config file location, number of replicas, script file name."
fi

CONFIG_FILE=$1
REPLICATION_FACTOR=$2
COORD_SCRIPT_FILENAME=$3

echo "Using config file located at $CONFIG_FILE"
echo "Using script: $COORD_SCRIPT_FILENAME"
source $CONFIG_FILE

if [ $? -ne 0 ]; then 
	echo "Config file at $CONFIG_FILE not found."
	exit 1
fi	



echo ">>>> Replication factor to be used: $REPLICATION_FACTOR"
echo ">>>> Number of replicas to be created: $NUMBER_OF_MACHINES_NEEDED"

COORDINATOR_SCRIPT_PATH="scripts/"

COORDINATOR_SCRIPT_LOCATION="$COORDINATOR_SCRIPT_PATH/$COORD_SCRIPT_FILENAME"

echo "Using co-ordinator script located at: $COORDINATOR_SCRIPT_LOCATION"

echo "Remote instances to be used: ${ALL_REMOTE_INSTANCES[@]:0:$NUMBER_OF_MACHINES_NEEDED}"

HOSTNAME=`hostname`
LIST_OF_ALL_INSTANCES="$HOSTNAME ${ALL_REMOTE_INSTANCES[@]}"

echo "List of all instances: $LIST_OF_ALL_INSTANCES"


#################################################################################
#Get the latest version of H2O:

../h2o/clearlogs.sh
../h2o/deletedbfiles.sh
../h2o/getlatest.sh

#################################################################################
#Kill any previous worker nodes:

pkill -9 -f '.*h2o.*'

for REMOTE_INSTANCE in "${ALL_REMOTE_INSTANCES[@]}"
do
	echo "Killing any existing H2O evaluation instances on $REMOTE_INSTANCE."

	( # Start of subshell
		ssh root@$REMOTE_INSTANCE "pkill -9 -f '.*h2o.*'"
	) &
done

wait

echo "Showing running H2O processes (there should be none):"
ps aux | grep h2o

#################################################################################
#Start worker nodes:

CLASS_TO_START="org.h2o.eval.madface.MadfaceWorkerStarter"

DEBUG_CODE="-Xdebug -Xrunjdwp:transport=dt_socket,address=8998,server=y"
CLASSPATH="bin/h2o.jar::bin/json.jar:bin/mailapi.jar:bin/mindterm.jar:bin/lucene-core-2.2.0.jar:bin/org.osgi.core-1.2.0.jar:bin/servlet-api-2.4.jar:bin/slf4j-api-1.5.0.jar:bin/sigar.jar"

LIST_OF_NEEDED_INSTANCES="$HOSTNAME ${ALL_REMOTE_INSTANCES[@]:0:$NUMBER_OF_MACHINES_NEEDED}"

if [ $WORKER_DEBUG == true ]; then
	java $DEBUG_CODE -cp $CLASSPATH $CLASS_TO_START "$LIST_OF_NEEDED_INSTANCES" $H2O_JAR_NAME
else 
	java -cp $CLASSPATH $CLASS_TO_START "$LIST_OF_NEEDED_INSTANCES" $H2O_JAR_NAME
fi

	if [ $? -ne 0 ]; then 
		echo "Error executing: java -cp $CLASSPATH $CLASS_TO_START '$LIST_OF_NEEDED_INSTANCES' $H2O_JAR_NAME"
		exit 1
	fi
	
#Note: this should return once all workers have been started.

###################################################################################
#Start co-ordinator node:

DATABASE_NAME=my_test_database
#let TOTAL_NUMBER_OF_MACHINES_NEEDED=$NUMBER_OF_MACHINES_NEEDED+1
TERMINATE_EXISTING=true
PROPERTIES_FILE_PATH=`pwd`/cnf/h2o.properties
CHAR_SEPARATED_HOSTNAMES="$HOSTNAME;"


for NEXT_HOSTNAME in "${ALL_REMOTE_INSTANCES[@]}"
do
CHAR_SEPARATED_HOSTNAMES="$CHAR_SEPARATED_HOSTNAMES;$NEXT_HOSTNAME"
done

echo "CHAR_SEPARATED_HOSTNAMES = $CHAR_SEPARATED_HOSTNAMES"


CLASS_TO_START="org.h2o.eval.CoordinationScriptRunner"
ARGS="-n$DATABASE_NAME -c$COORDINATOR_SCRIPT_LOCATION -f$RESULTS_FOLDER_LOCATION -r$REPLICATION_FACTOR -w$CHAR_SEPARATED_HOSTNAMES -t$RESULTS_TIME_SPLICE"
	
CLASSPATH="bin/h2o.jar::bin/json.jar:bin/mailapi.jar:bin/mindterm.jar:bin/lucene-core-2.2.0.jar:bin/org.osgi.core-1.2.0.jar:bin/servlet-api-2.4.jar:bin/slf4j-api-1.5.0.jar:bin/sigar.jar"
	
if [ $COORD_DEBUG == true ]; then
	java $DEBUG_CODE -cp $CLASSPATH $CLASS_TO_START $ARGS
else 
	java -cp $CLASSPATH $CLASS_TO_START $ARGS
fi

if [ $? -ne 0 ]; then 
	echo "Error executing: java -cp $CLASSPATH $CLASS_TO_START $ARGS"
	exit 2
fi	

#By this point the co-ordinator script should have started 
