#!/bin/bash
# License: BSD License

# As Example we start 3 production processes in the background
# always record PID in PRODPID 

# 1. Start new process for test
# 2. Record PID in PRODPID.
# 3. exit_timeout function called on timeout. This should kill the process then return an error code to the executing script requiring retry.

echo "Starting timeout script..."
./_executeScript.sh $1 $2 $3 &
echo "Timeout script started, moving on to sleep..."

PRODPID[1]=$!

export PRODPID


# record own PID
export PID=$$

# define exit function
exit_timeout() {
  echo "Timeout. These processes are not finished:"
  for i in ${PRODPID[@]} ; do
    ps -p $i |grep -v "PID TTY      TIME CMD"
    if [ $? == 0 ] ; then
      # process still alive
      echo "Sending SIGTERM to process $i"
      	pkill -9 -f '.*org.h2o.*'
	kill $i
echo "Killed!"	  
	  exit 5 #this assumes that there is only one process in PRODPID - a hack.
    fi
  done
  # timeout CLEAN exit
  exit
}

# Handler for signal USR1 for the timer
trap exit_timeout SIGUSR1

# starting timer in subshell. It sends a SIGUSR1 to the father if it timeouts.
export TIMEOUT=3600
echo "About to sleep..."
(sleep $TIMEOUT ; kill -SIGUSR1 $PID) &

# record PID of timer
TPID=$!


# wait for all production processes to finish
wait ${PRODPID[*]}

# Normal exit
echo "All processes ended normal"

# kill timer
kill $TPID
