#!/bin/bash

H2O_JAR_NAME="h2o.jar"

WORKER_DEBUG=false
COORD_DEBUG=false
NEW_CNF_FILE=""

SCRIPT_CONFIG_FILE=user-config.cfg
NUMBER_OF_REPLICAS=""
if [ $# == "3" ]; then
	NEW_CNF_FILE=$1 #not used in H2O. This parameter is kept here to keep it consistent with the others.
	let NUMBER_OF_REPLICAS=$3-1 #the main instance counts as a replica. this is used to calc. the number of workers that will come from the ALL_REMOTE_INSTANCES array.
	SCRIPT_CONFIG_FILE=$2 #benchmarking config. file.	
fi

source $SCRIPT_CONFIG_FILE

echo "STATIC_CONFIG_FILE: $STATIC_CONFIG_FILE"
source $STATIC_CONFIG_FILE

echo ">>>> Number of replicas to be created: $NUMBER_OF_REPLICAS"
echo ">>>> Config file being used: $SCRIPT_CONFIG_FILE"

echo "Remote instances to be used: ${ALL_REMOTE_INSTANCES[@]:0:$NUMBER_OF_REPLICAS}"

HOSTNAME=`hostname`
LIST_OF_ALL_INSTANCES="$HOSTNAME ${ALL_REMOTE_INSTANCES[@]}"

echo "List of all instances: $LIST_OF_ALL_INSTANCES"


#################################################################################
#Get the latest version of H2O:

./clearlogs.sh
./getlatest.sh

#################################################################################
#Kill any previous worker nodes:

pkill -9 -f '.*h2o.*'

for REMOTE_INSTANCE in "${ALL_REMOTE_INSTANCES[@]}"
do
	echo "Killing any existing H2O evaluation instances on $REMOTE_INSTANCE."

	( # Start of subshell
		ssh root@$REMOTE_INSTANCE "pkill -9 -f '.*h2o.*'"
	) &
done

wait

echo "Showing running H2O processes (there should be none):"
ps aux | grep h2o

#################################################################################
#Start worker nodes:

CLASS_TO_START="org.h2o.eval.madface.MadfaceWorkerStarter"

DEBUG_CODE="-Xdebug -Xrunjdwp:transport=dt_socket,address=8998,server=y"
CLASSPATH="bin/h2o.jar::bin/json.jar:bin/mailapi.jar:bin/mindterm.jar:bin/lucene-core-2.2.0.jar:bin/org.osgi.core-1.2.0.jar:bin/servlet-api-2.4.jar:bin/slf4j-api-1.5.0.jar:bin/sigar.jar"

LIST_OF_NEEDED_INSTANCES="$HOSTNAME ${ALL_REMOTE_INSTANCES[@]:0:$NUMBER_OF_REPLICAS}"


if [ $WORKER_DEBUG == true ]; then
	java $DEBUG_CODE -cp $CLASSPATH $CLASS_TO_START "$LIST_OF_NEEDED_INSTANCES" $H2O_JAR_NAME
else 
	java -cp $CLASSPATH $CLASS_TO_START "$LIST_OF_NEEDED_INSTANCES" $H2O_JAR_NAME
fi

	if [ $? -ne 0 ]; then 
		echo "Error executing: java -cp $CLASSPATH $CLASS_TO_START \"$LIST_OF_NEEDED_INSTANCES\" $H2O_JAR_NAME"
		exit 1
	fi
	
#Note: this should return once all workers have been started.

###################################################################################
#Start co-ordinator node:

DATABASE_NAME=my_test_database
let TOTAL_NUMBER_OF_REPLICAS=$NUMBER_OF_REPLICAS+1
TERMINATE_EXISTING=true
PROPERTIES_FILE_PATH=`pwd`/cnf/h2o.properties
CHAR_SEPARATED_HOSTNAMES="$HOSTNAME;"


for NEXT_HOSTNAME in "${ALL_REMOTE_INSTANCES[@]}"
do
CHAR_SEPARATED_HOSTNAMES="$CHAR_SEPARATED_HOSTNAMES;$NEXT_HOSTNAME"
done

echo "CHAR_SEPARATED_HOSTNAMES = $CHAR_SEPARATED_HOSTNAMES"


CLASS_TO_START="org.h2o.eval.Coordinator"
ARGS="-n$DATABASE_NAME -c$TOTAL_NUMBER_OF_REPLICAS -r$TOTAL_NUMBER_OF_REPLICAS -t$TERMINATE_EXISTING -p$PROPERTIES_FILE_PATH -w$CHAR_SEPARATED_HOSTNAMES"
	
CLASSPATH="bin/h2o.jar::bin/json.jar:bin/mailapi.jar:bin/mindterm.jar:bin/lucene-core-2.2.0.jar:bin/org.osgi.core-1.2.0.jar:bin/servlet-api-2.4.jar:bin/slf4j-api-1.5.0.jar:bin/sigar.jar"
	
	
if [ $COORD_DEBUG == true ]; then
	java $DEBUG_CODE -cp $CLASSPATH $CLASS_TO_START $ARGS
else 
	java -cp $CLASSPATH $CLASS_TO_START $ARGS
fi

if [ $? -ne 0 ]; then 
	echo "Error executing: java -cp $CLASSPATH $CLASS_TO_START $ARGS"
	exit 1
fi	

#By this point the co-ordinator should have started H2O instances on the specified number of nodes, so the benchmark can now be executed.
