#!/bin/bash


source $1
CONFIG=$2 #used for logging
NUMBER_OF_REPLICAS=$3 #used for logging
DATABASE_CONNECTION_INFO=$4
DATABASE_NAME=$5
MINUTES_TO_EXECUTE=$6
###################################
# LOGGING / ERROR FUNCTIONS

#Lock file and logging details
#NOTE: The script records completed tasks in the following log file; the file has a secondary function as a lock file.
# Upon completion (successful or insuccessful) of the script the log file will be removed. Prior to this its contents will be emailed to the relevant address below.

HOSTNAME=`hostname`
DATE_TIME=`date +%F@%H-%M` # the current date and time

LOGFILE=/tmp/$DATABASE_NAME-benchmark.log

#diagmail: two arguments; the message to be mailed, the subject line of the email.
# Sends an email to the SUCCESS_MAILTARGET address informing the person of the scripts completion, and appending the contents of the logfile.
function diagmail {
	echo "$1" | mail -s "$2" $SUCCESS_MAILTARGET
}  

# diagfail: one argument: error message to be mailed.
# Emails the appropriate alias to inform them of the failure of the backup, and the reason why (which is provided as an argument).
function diagfail {
	
	echo "$1
	
Logs prior to failure:

`cat $LOGFILE`" | mail -s "[Eval] FAILURE. $DATABASE_NAME benchmark on $HOSTNAME FAILED at $DATE_TIME" $FAILURE_MAILTARGET 

	rm -f $LOGFILE
	trap - INT TERM EXIT #Remove the traps to the following system calls.
	exit 1
}

# toLog: one argument, text to be written to the log file.
# Pre-pend the function argument with the current date and time, then add it to the log file.
function tolog {
	echo "`date +%F@%H-%M-%S`: $1
	" >> $LOGFILE
}
############################################################

#Trap system calls related to the scripts premature exit. Prior to this exit ensure the log file's contents are emailed to the appropriate person.
trap "tolog 'Script unexpectedly called an OS exit command. Performing clean shutdown.';diagfail 'Script unexpectedly quit.'; exit" INT TERM EXIT
#Create a log file
touch $LOGFILE
echo "" > $LOGFILE #ensure it is empty.
tolog "Log file created at $LOGFILE"

################################################################


if [ $CREATE_DB_TABLES == true ] ; then
	
	$JAVA_HOME/bin/java -cp $BENCHMARKSQL_DIR/lib/edb-jdbc14-8_0_3_14.jar:$BENCHMARKSQL_DIR/lib/ojdbc14-10.2.jar:$BENCHMARKSQL_DIR/lib/postgresql-8.0.309.jdbc3.jar:$BENCHMARKSQL_DIR/lib/db2jcc.jar:$BENCHMARKSQL_DIR/dist/BenchmarkSQL-2.3.jar:$BENCHMARKSQL_DIR/lib/h2o.jar:$BENCHMARKSQL_DIR/lib/mysql-connector-java.jar -Dprop=$DATABASE_CONNECTION_INFO -DcommandFile=$BENCHMARKSQL_DIR/run/sqlTableCreates ExecJDBC 

	if [ $? -ne 0 ]; then 
			diagfail "Failed to create tables on $HOSTNAME, with configuration '$DATABASE_CONNECTION_INFO'."
			exit 1
		else		
			tolog "Successfully created benchmarking tables on $HOSTNAME, with configuration '$DATABASE_CONNECTION_INFO'."
	fi		
	
	$JAVA_HOME/bin/java -cp $BENCHMARKSQL_DIR/lib/edb-jdbc14-8_0_3_14.jar:$BENCHMARKSQL_DIR/lib/ojdbc14-10.2.jar:$BENCHMARKSQL_DIR/lib/postgresql-8.0.309.jdbc3.jar:$BENCHMARKSQL_DIR/lib/db2jcc.jar:$BENCHMARKSQL_DIR/dist/BenchmarkSQL-2.3.jar:$BENCHMARKSQL_DIR/lib/h2o.jar:$BENCHMARKSQL_DIR/lib/mysql-connector-java.jar  -Dprop=$DATABASE_CONNECTION_INFO LoadData -log$RESULTS_DIRECTORY -c$CONFIG -r$NUMBER_OF_REPLICAS -h$HOT_START

	if [ $? -ne 0 ]; then 
			diagfail "Failed to load data."
			exit 1
		else		
			tolog "Successfully loaded data into benchmarking tables.."
	fi		
	
	$JAVA_HOME/bin/java -cp $BENCHMARKSQL_DIR/lib/edb-jdbc14-8_0_3_14.jar:$BENCHMARKSQL_DIR/lib/ojdbc14-10.2.jar:$BENCHMARKSQL_DIR/lib/postgresql-8.0.309.jdbc3.jar:$BENCHMARKSQL_DIR/lib/db2jcc.jar:$BENCHMARKSQL_DIR/dist/BenchmarkSQL-2.3.jar:$BENCHMARKSQL_DIR/lib/h2o.jar:$BENCHMARKSQL_DIR/lib/mysql-connector-java.jar -Dprop=$DATABASE_CONNECTION_INFO -DcommandFile=$BENCHMARKSQL_DIR/run/sqlIndexCreates ExecJDBC 
fi

#Perform single hot start (non-recorded benchmark) if specified:
if [ $HOT_START == true ] ; then
	echo "Performing initial benchmark for hot start"
	$JAVA_HOME/bin/java -cp $BENCHMARKSQL_DIR/lib/edb-jdbc14-8_0_3_14.jar:$BENCHMARKSQL_DIR/lib/ojdbc14-10.2.jar:$BENCHMARKSQL_DIR/lib/postgresql-8.0.309.jdbc3.jar:$BENCHMARKSQL_DIR/lib/db2jcc.jar:$BENCHMARKSQL_DIR/dist/BenchmarkSQL-2.3.jar:$BENCHMARKSQL_DIR/lib/h2o.jar:$BENCHMARKSQL_DIR/lib/mysql-connector-java.jar -Dprop=$DATABASE_CONNECTION_INFO jTPCC -silent -m2

	if [ $? -ne 0 ]; then 
		diagfail "Failed to perform hot start benchmark."
		exit 1
	else		
		tolog "Successfully performed hot start benchmark."
	fi		
fi


	$JAVA_HOME/bin/java -cp $BENCHMARKSQL_DIR/lib/edb-jdbc14-8_0_3_14.jar:$BENCHMARKSQL_DIR/lib/ojdbc14-10.2.jar:$BENCHMARKSQL_DIR/lib/postgresql-8.0.309.jdbc3.jar:$BENCHMARKSQL_DIR/lib/db2jcc.jar:$BENCHMARKSQL_DIR/dist/BenchmarkSQL-2.3.jar:$BENCHMARKSQL_DIR/lib/h2o.jar:$BENCHMARKSQL_DIR/lib/mysql-connector-java.jar -Dprop=$DATABASE_CONNECTION_INFO jTPCC -log$RESULTS_DIRECTORY -c$CONFIG -r$NUMBER_OF_REPLICAS -h$HOT_START -m$MINUTES_TO_EXECUTE -stdout

	if [ $? -ne 0 ]; then 
		diagfail "Failed to perform benchmark number $c."
		exit 1
	else		
		tolog "Successfully performed benchmark number $c in $MINUTES_TO_EXECUTE minutes."
	fi	


#########################################################################################################
#Send log at end of evaluation:

if [ -f $LOGFILE ]; then
	diagmail "Completed execution of benchmarks. Settings used:
	- BenchmarkSQL location: $BENCHMARKSQL_DIR
	- Config file location: $DATABASE_CONNECTION_INFO
	- Logging location: $RESULTS_DIRECTORY
	- Configuration of database: $CONFIG
	- Tables and data added immediately before evaluation: $CREATE_TABLES_AND_DATA
	- Hot start enabled: $HOT_START
	- Evaluation period: $MINUTES minutes
	- Number of evaluations run: $NUM_OF_RUNS

	
Logs of this script's execution:

`cat $LOGFILE`" "[Eval] SUCCESS. Completed MySQL evaluations of $DATABASE_CONNECTION_INFO."

	trap - INT TERM EXIT #Remove the traps to the following system calls.

	echo "Script has successfully terminated."
fi



