#!/bin/bash

if [ $# != "4" ]; then
    echo "Not enough parameters;"
    echo "Parameter 1: the name of the master host (e.g. beast.cs.st-andrews.ac.uk)"
	echo "Parameter 2: the name of the binary log file on the master (e.g. mysql-bin.000003). Found using the mysql-master-setup script, or 'SHOW MASTER STATUS' on the master."
	echo "Parameter 3: the current position in the binary log file (e.g. 488). Found using the mysql-master-setup script, or 'SHOW MASTER STATUS' on the master."
	echo "Parameter 4: Name of the config file to be used."
	
	echo "For parameters 2 and 3, if there is no existing binary log file the correct values are: \"('')\" and \"4\"."
	
	exit
fi

NON_STATIC_CONFIG=$4

source /tmp/$NON_STATIC_CONFIG
source /tmp/static-config.cfg

echo "Using config files '$NON_STATIC_CONFIG' and 'static-config.cfg'"

MASTER_HOST=$1
MASTER_LOG_FILE=$2
MASTER_LOG_POS=$3

/etc/init.d/mysqld stop 1> /dev/null #in-case it's already installed.

#Install clean MySQL Version:
yum -q remove mysql 1> /dev/null
yum -q install mysql-server 1> /dev/null
yum -q install mysql-devel 1> /dev/null

/etc/init.d/mysqld start > /dev/null

#Set root password
echo "Setting MySQL root $MYSQL_USERNAME on $HOSTNAME."
/usr/bin/mysqladmin -s -u $MYSQL_USERNAME password "$MYSQL_PASSWORD"

#Set up the MySQL database instance.

for i in "${TO_REMOVE[@]}"
do
    sed -i "s/^$i.*//" "$MYSQL_CONFIG_FILE"
done

REGEX="compute-0-([0-9][0-9]?).local" #To match the hostname and get a unique number from it.
HOSTNAME=`hostname`

ID="-1" #It will be changed from this.
#Get the ID for this MySQL instance based on the hostname of the server.
if [[ "$HOSTNAME" =~ "$REGEX" ]]
then
	n=${#BASH_REMATCH[*]}
	ID="${BASH_REMATCH[1]}"
else
	echo "WARNING: didn't find a match on the hostname '$HOSTNAME'"
fi

ID_LINE="server-id=$ID"
MASTER_HOST_LINE="master-host=$MASTER_HOST"
MASTER_USER_LINE="master-user=$REPLICATION_USER"
MASTER_PASSWORD_LINE="master-password=$REPLICATION_PASSWORD"
MASTER_CONNECT_RETRY_LINE="master-connect-retry=60"
DATABASE_TO_REPLICATE_LINE="replicate-do-db=$DATABASE_TO_REPLICATE"

sed -i "s/\[mysqld\]/\[mysqld\]\n$ID_LINE\n$MASTER_HOST_LINE\n$MASTER_USER_LINE\n$MASTER_PASSWORD_LINE\n$MASTER_CONNECT_RETRY_LINE\n$DATABASE_TO_REPLICATE_LINE\n/" "$MYSQL_CONFIG_FILE"

#Restart the MySQL instance

/etc/init.d/mysqld restart > /dev/null

MYSQL="mysql -u$MYSQL_USERNAME -p$MYSQL_PASSWORD -e"

#Initiate connection to the master...
`$MYSQL "STOP SLAVE;"` #prevents an error on the next commands if one happens to be running.

`$MYSQL "DROP DATABASE IF EXISTS $DATABASE_TO_REPLICATE;"` #clean any previous install.
`$MYSQL "CREATE DATABASE $DATABASE_TO_REPLICATE;"` #create the database that is to be replicated.

`$MYSQL "CHANGE MASTER TO MASTER_HOST='$MASTER_HOST', MASTER_USER='$REPLICATION_USER', MASTER_PASSWORD='$REPLICATION_PASSWORD', MASTER_LOG_FILE='$MASTER_LOG_FILE', MASTER_LOG_POS=$MASTER_LOG_POS;"`
`$MYSQL "START SLAVE;"`

